process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0'
process.env.SUPPRESS_NO_CONFIG_WARNING = true

const httpMocks = require('node-mocks-http')
const fixtures = require('./../fixtures/fixtures')
const Model = require('./../helpers/Model')
const Chance = require('chance')

const chance = new Chance()

const models = Model.build()
jest.mock('pubsweet-component-mail-service', () => ({
  setupAssignEmail: jest.fn(),
  setupHandlingEditorAgreedEmail: jest.fn(),
  setupDeclineEmail: jest.fn(),
}))

const { author, submittingAuthor } = fixtures.users
const { standardCollection, authorsCollection } = fixtures.collections
const body = {
  isSubmitting: false,
  isCorresponding: true,
  firstName: chance.first(),
  lastName: chance.last(),
  affiliation: chance.company(),
}
const patchPath = '../../routes/collectionsUsers/patch'
describe('Patch collections users route handler', () => {
  it('should return success when the request data is correct', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    req.params.userId = submittingAuthor.id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)
    const data = JSON.parse(res._getData())
    expect(res.statusCode).toBe(200)
    const matchingAuthor = data.authors.find(
      author => author.userId === submittingAuthor.id,
    )
    expect(matchingAuthor.isSubmitting).toBe(body.isSubmitting)
    expect(matchingAuthor.isCorresponding).toBe(body.isCorresponding)
    expect(submittingAuthor.firstName).toBe(body.firstName)
    expect(submittingAuthor.lastName).toBe(body.lastName)
  })
  it('should return an error when the params are missing', async () => {
    delete body.isSubmitting
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    req.params.userId = submittingAuthor.id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Missing parameters')
    body.isSubmitting = false
  })
  it('should return an error if the collection does not exists', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = 'invalid-id'
    req.params.userId = submittingAuthor.id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(404)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('item not found')
  })
  it('should return an error when the user does not exist', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = author.id
    req.params.collectionId = standardCollection.id
    req.params.userId = 'invalid-id'
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(404)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('item not found')
  })
  it('should return an error when the collection does not have authors', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = authorsCollection.id
    req.params.userId = submittingAuthor.id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Collection does not have any authors')
  })
  it('should return an error when the collection and the user do not match', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    req.params.userId = author.id
    const res = httpMocks.createResponse()
    await require(patchPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Collection and user do not match')
  })
})
