process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0'
process.env.SUPPRESS_NO_CONFIG_WARNING = true

const httpMocks = require('node-mocks-http')
const fixtures = require('./../fixtures/fixtures')
const Chance = require('chance')
const Model = require('./../helpers/Model')

const models = Model.build()
jest.mock('pubsweet-component-mail-service', () => ({
  setupInviteEmail: jest.fn(),
  setupAssignEmail: jest.fn(),
}))
const chance = new Chance()

const { author, submittingAuthor } = fixtures.users
const { standardCollection } = fixtures.collections
const postPath = '../../routes/collectionsUsers/post'
const body = {
  email: chance.email(),
  role: 'author',
  isSubmitting: true,
  isCorresponding: false,
}
describe('Post collections users route handler', () => {
  it('should return success when an author adds a new user to a collection', async () => {
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    const res = httpMocks.createResponse()
    await require(postPath)(models)(req, res)

    expect(res.statusCode).toBe(200)
    const data = JSON.parse(res._getData())
    expect(data.email).toEqual(body.email)
    expect(data.invitations).toBeUndefined()
    const matchingAuthor = standardCollection.authors.find(
      author => author.userId === data.id,
    )
    expect(matchingAuthor).toBeDefined()
  })
  it('should return success when an author adds an existing user as co author to a collection', async () => {
    body.email = author.email
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    const res = httpMocks.createResponse()
    await require(postPath)(models)(req, res)

    expect(res.statusCode).toBe(200)
    const data = JSON.parse(res._getData())
    expect(data.email).toEqual(body.email)
    const matchingAuthor = standardCollection.authors.find(
      auth => auth.userId === author.id,
    )
    expect(matchingAuthor).toBeDefined()
  })
  it('should return an error when the an author is added to the same collection', async () => {
    body.email = submittingAuthor.email
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    const res = httpMocks.createResponse()
    await require(postPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual(
      `User ${submittingAuthor.email} is already an author`,
    )
  })
  it('should return an error when params are missing', async () => {
    delete body.email
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    const res = httpMocks.createResponse()
    await require(postPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('Missing parameters.')
    body.email = author.email
  })
  it('should return an error when the role is not author and the user exists', async () => {
    body.role = 'invalid-role'
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    const res = httpMocks.createResponse()
    await require(postPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('invalid-role is not defined')
  })
  it('should return an error when the role is not author and the user is new', async () => {
    body.role = 'invalid-role'
    body.email = chance.email()
    const req = httpMocks.createRequest({
      body,
    })
    req.user = submittingAuthor.id
    req.params.collectionId = standardCollection.id
    const res = httpMocks.createResponse()
    await require(postPath)(models)(req, res)

    expect(res.statusCode).toBe(400)
    const data = JSON.parse(res._getData())
    expect(data.error).toEqual('invalid-role is not defined')
  })
})
